/*
 * Copyright (c) 2008, Harald Kuhr
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name "TwelveMonkeys" nor the
 *       names of its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package com.twelvemonkeys.imageio.plugins.ico;

import javax.imageio.ImageReader;
import javax.imageio.spi.ImageReaderSpi;
import javax.imageio.stream.ImageInputStream;
import java.io.IOException;
import java.util.Locale;

/**
 * ICOImageReaderSpi class description.
 *
 * @author <a href="mailto:harald.kuhr@gmail.com">Harald Kuhr</a>
 * @version $Id: ICOImageReaderSpi.java,v 1.0 25.feb.2006 00:29:44 haku Exp$
 */
public class ICOImageReaderSpi extends ImageReaderSpi {

    public ICOImageReaderSpi() {
        super(
                "TwelveMonkeys",
                "2.0",
                new String[]{"ico", "ICO"},
                new String[]{"ico"},
                new String[]{
                        "image/vnd.microsoft.icon", // Official IANA MIME
                        "image/x-icon",             // Common extension MIME
                        "image/ico"                 // Unofficial, but common
                },
                "com.twelvemonkeys.imageio.plugins.ico.ICOImageReader",
                STANDARD_INPUT_TYPE,
                null,
                true, null, null, null, null,
                true,
                "com.twelvemonkeys.imageio.plugins.ico.ICOMetadata_1.0",
                "com.twelvemonkeys.imageio.plugins.ico.ICOMetadata",
                null, null
        );
    }

    public boolean canDecodeInput(Object pSource) throws IOException {
        return pSource instanceof ImageInputStream && canDecode((ImageInputStream) pSource);
    }

    static boolean canDecode(ImageInputStream pInput) throws IOException {
        byte[] signature = new byte[4];

        try {
            pInput.mark();
            pInput.readFully(signature);

            return (signature[0] == 0x0 && signature[1] == 0x0 && signature[2] == 0x1 && signature[3] == 0x0);
        }
        finally {
            pInput.reset();
        }
    }

    public ImageReader createReaderInstance(Object pExtension) throws IOException {
        return new ICOImageReader(this);
    }

    public String getDescription(Locale pLocale) {
        return "Windows Icon Format (ICO) Reader";
    }
}
